/* IBM_COPYRIGHT_BEGIN_TAG */
/*
 * Copyright IBM Corporation 2009. All rights reserved. The information contained
 * in these materials is provided for informational purposes only, and is provided AS IS
 * without warranty of any kind, express or implied. IBM shall not be responsible for any
 * damages arising out of the use of, or otherwise related to, these materials. Nothing
 * contained in these materials is intended to, nor shall have the effect of, creating any
 * warranties or representations from IBM or its suppliers or licensors, or altering the terms
 * and conditions of the applicable license agreement governing the use of IBM software.
 * References in these materials to IBM products, programs, or services do not imply that
 * they will be available in all countries in which IBM operates. Product release dates
 * and/or capabilities referenced in these materials may change at any time at IBM<92>s sole
 * discretion based on market opportunities or other factors, and are not intended to be a
 * commitment to future product or feature availability in any way. IBM, the IBM logo, AIX,
 * the AIX logo, and other IBM products and services are trademarks of the International
 * Business Machines Corporation, in the United States, other countries or both. Other
 * company, product, or service names may be trademarks or service marks of others.
 */
/* IBM_COPYRIGHT_END_TAG */

#include <pthread.h>
#include <stdio.h>
#include "Thread.h"
#include "Mutex.h"
#include "Job.h"
#include "Step.h"
#include <stdint.h>
#include "llrapi.h"
#include "rm_symbols.h"
#include "Scheduler.h"
#include "Log.h"

void
Job::clear_steps() {
	for (int i=0; i<steps.size(); i++) {
		steps[i]->rel_ref();
	}

	steps.clear();
}

bool
Job::updateData(void* llr_job) {
	Scheduler&     s   = Scheduler::getInstance();
	llr_element_t* err = NULL;
	int            rc  = 0;

	GET_DATA_P(rc, s.rm_handle, llr_job, LLR_JobName, JobName, err);
	GET_DATA(rc, s.rm_handle, llr_job, LLR_JobMode, Mode, err);
	GET_DATA_P(rc, s.rm_handle, llr_job, LLR_JobSubmitHost, SubmitHost, err);
	GET_DATA(rc, s.rm_handle, llr_job, LLR_JobJobQueueKey, JobQueueKey, err);
	GET_DATA_P(rc, s.rm_handle, llr_job, LLR_JobJobManager, JobManager, err);
	GET_DATA(rc, s.rm_handle, llr_job, LLR_JobSubmitTime, SubmitTime, err);
	GET_DATA(rc, s.rm_handle, llr_job, LLR_JobVersionNum, VersionNum, err);
	GET_DATA(rc, s.rm_handle, llr_job, LLR_JobStepCount, StepCount, err);

	llr_data_list_t llr_step_list;
	void* llr_step = NULL;
	int i = 0;
	GET_DATA(rc, s.rm_handle, llr_job, LLR_JobGetStepList, llr_step_list, err);
	GET_DATA(rc, s.rm_handle, &llr_step_list, LLR_JobGetFirstStep, llr_step, err);
	while (llr_step != NULL) {
		err = NULL;
		char* step_id = NULL;
		GET_DATA(rc, s.rm_handle, llr_step, LLR_StepID, step_id, err);
		if (!err) {
			log(1, "Step[%d] = %s (%p)\n", i++, step_id, llr_step);
			Step* step = s.findStep(step_id);
			bool need_to_add = false;
			if (step == NULL) {
				step = new Step(step_id);
				need_to_add = true;
			}
			step->get_ref();
			if (need_to_add) {
				steps.push_back(step);
			}
			step->lock.lock();
			step->job(this);
			step->updateData(llr_step);
			step->lock.unlock();
			if (need_to_add) {
				s.addStep(step);
			}
		}

		FREE_IF_SET(step_id);
		GET_DATA(rc, s.rm_handle, &llr_step_list, LLR_JobGetNextStep, llr_step, err);
	}

	display_job_data();
	return (rc == 0);
}

void
Job::display_job_data() {
	log(1, "JOB JobID:                       %s\n", JobID);
	log(2, "JOB JobName:                     %s\n", JobName);
	log(2, "JOB Mode:                        %d\n", Mode);
	log(2, "JOB SubmitHost:                  %s\n", SubmitHost);
	log(2, "JOB JobQueueKey:                 %d\n", JobQueueKey);
	log(2, "JOB JobManager:                  %s\n", JobManager);
	log(2, "JOB SubmitTime:                  %d\n", SubmitTime);
	log(2, "JOB VersionNum:                  %d\n", VersionNum);
	log(2, "JOB StepCount:                   %d\n", StepCount);
}

Job::Job(char* job_id) {
        JobID       = strdup(job_id);
        JobName     = NULL;
        Mode        = 0;
        SubmitHost  = NULL;
        JobQueueKey = 0;
        JobManager  = NULL;
        SubmitTime  = 0;
        VersionNum  = 0;
        StepCount   = 0;
}

Job::~Job() {
	clear_steps();
	FREE_IF_SET(JobName);
	FREE_IF_SET(JobID);
	FREE_IF_SET(SubmitHost);
	FREE_IF_SET(JobManager);
}
