/* IBM_COPYRIGHT_BEGIN_TAG */
/*
 * Copyright IBM Corporation 2009. All rights reserved. The information contained
 * in these materials is provided for informational purposes only, and is provided AS IS
 * without warranty of any kind, express or implied. IBM shall not be responsible for any
 * damages arising out of the use of, or otherwise related to, these materials. Nothing
 * contained in these materials is intended to, nor shall have the effect of, creating any
 * warranties or representations from IBM or its suppliers or licensors, or altering the terms
 * and conditions of the applicable license agreement governing the use of IBM software.
 * References in these materials to IBM products, programs, or services do not imply that
 * they will be available in all countries in which IBM operates. Product release dates
 * and/or capabilities referenced in these materials may change at any time at IBM<92>s sole
 * discretion based on market opportunities or other factors, and are not intended to be a
 * commitment to future product or feature availability in any way. IBM, the IBM logo, AIX,
 * the AIX logo, and other IBM products and services are trademarks of the International
 * Business Machines Corporation, in the United States, other countries or both. Other
 * company, product, or service names may be trademarks or service marks of others.
 */
/* IBM_COPYRIGHT_END_TAG */

#include <pthread.h>
#include <stdio.h>
#include "Thread.h"
#include "Mutex.h"
#include "Node.h"
#include <stdint.h>
#include "llrapi.h"
#include "rm_symbols.h"
#include "Scheduler.h"
#include "Log.h"

bool
Node::updateData(void* llr_node) {
	Scheduler&     s   = Scheduler::getInstance();
	llr_element_t* err = NULL;
	int            rc  = 0;

	GET_DATA(rc, s.rm_handle, llr_node, LLR_NodeTaskCount, TaskCount, err);
	GET_DATA(rc, s.rm_handle, llr_node, LLR_NodeMaxInstances, MaxInstances, err);
	GET_DATA(rc, s.rm_handle, llr_node, LLR_NodeMinInstances, MinInstances, err);
	GET_DATA_P(rc, s.rm_handle, llr_node, LLR_NodeRequirements, Requirements, err);
	GET_DATA(rc, s.rm_handle, llr_node, LLR_NodeInitiatorCount, InitiatorCount, err);

	display_node_data();
	return (rc == 0);
}

void
Node::display_node_data() {
	log(2, "NODE InitiatorCount:             %d\n", InitiatorCount);
	log(2, "NODE Requirements:               %s\n", Requirements);
	log(2, "NODE MaxInstances:               %d\n", MaxInstances);
	log(2, "NODE MinInstances:               %d\n", MinInstances);
	log(2, "NODE TaskCount:                  %d\n", TaskCount);
}

Node::Node() {
	TaskCount      = 0;
	MaxInstances   = 0;
	MinInstances   = 0;
	Requirements   = NULL;
	InitiatorCount = 0;
}

Node::~Node() {
	FREE_IF_SET(Requirements);
}




