/* IBM_COPYRIGHT_BEGIN_TAG */
/*
 * Copyright IBM Corporation 2009. All rights reserved. The information contained
 * in these materials is provided for informational purposes only, and is provided AS IS
 * without warranty of any kind, express or implied. IBM shall not be responsible for any
 * damages arising out of the use of, or otherwise related to, these materials. Nothing
 * contained in these materials is intended to, nor shall have the effect of, creating any
 * warranties or representations from IBM or its suppliers or licensors, or altering the terms
 * and conditions of the applicable license agreement governing the use of IBM software.
 * References in these materials to IBM products, programs, or services do not imply that
 * they will be available in all countries in which IBM operates. Product release dates
 * and/or capabilities referenced in these materials may change at any time at IBM<92>s sole
 * discretion based on market opportunities or other factors, and are not intended to be a
 * commitment to future product or feature availability in any way. IBM, the IBM logo, AIX,
 * the AIX logo, and other IBM products and services are trademarks of the International
 * Business Machines Corporation, in the United States, other countries or both. Other
 * company, product, or service names may be trademarks or service marks of others.
 */
/* IBM_COPYRIGHT_END_TAG */

#include <pthread.h>
#include <stdio.h>
#include <unistd.h>
#include "Mutex.h"
#include "Log.h"
#include "RMResetThread.h"
#include "EventListenerThread.h"
#include "Scheduler.h"

void
RMResetThread::run() {
        Scheduler& scheduler = Scheduler::getInstance();

	// _rm_state_lock is used to make sure that reset and mark_down
	// are not taking place at the same time
	scheduler._rm_state_lock.lock();
	if (scheduler.RMisUP()) {
		log(0, "RM_RESET_THREAD: RM is UP, so need to first unregister from the RM\n");
		bool success = scheduler.unregister_with_rm();
		scheduler.RMisUP(false);
		if (success) {
			log(0, "RM_RESET_THREAD: successfully unregistered from the RM\n");
		} else {
			log(0, "RM_RESET_THREAD: could not unregister from the RM\n");
		}
	} else {
		log(0, "RM_RESET_THREAD: RM is not UP, no need to unregister from the RM\n");
	}

	bool success = scheduler.register_with_rm();

	if (success) {
		log(0, "RM_RESET_THREAD: successfully registered with the RM\n");
		scheduler.RMisUP(true);
		scheduler.eventListenerThread(new EventListenerThread());
		scheduler.eventListenerThread()->startThread();
	} else {
		log(0, "RM_RESET_THREAD: could not register with the RM\n");
		scheduler.RMisUP(false);
	}
	scheduler._rm_state_lock.unlock();

	scheduler.RMResetInProgress(false);
}
