/* IBM_COPYRIGHT_BEGIN_TAG */
/*
 * Copyright IBM Corporation 2009. All rights reserved. The information contained
 * in these materials is provided for informational purposes only, and is provided AS IS
 * without warranty of any kind, express or implied. IBM shall not be responsible for any
 * damages arising out of the use of, or otherwise related to, these materials. Nothing
 * contained in these materials is intended to, nor shall have the effect of, creating any
 * warranties or representations from IBM or its suppliers or licensors, or altering the terms
 * and conditions of the applicable license agreement governing the use of IBM software.
 * References in these materials to IBM products, programs, or services do not imply that
 * they will be available in all countries in which IBM operates. Product release dates
 * and/or capabilities referenced in these materials may change at any time at IBM<92>s sole
 * discretion based on market opportunities or other factors, and are not intended to be a
 * commitment to future product or feature availability in any way. IBM, the IBM logo, AIX,
 * the AIX logo, and other IBM products and services are trademarks of the International
 * Business Machines Corporation, in the United States, other countries or both. Other
 * company, product, or service names may be trademarks or service marks of others.
 */
/* IBM_COPYRIGHT_END_TAG */

#include <vector>
#include <list>
#include <stdint.h>
#include "llrapi.h"
#include "Job.h"
#include "Step.h"
#include "Machine.h"

#ifndef Scheduler_h
#define Scheduler_h

class SchedulingThread;
class EventListenerThread;

class Scheduler {
public:
	static Scheduler& getInstance() {
		if (_instance == NULL) {
			_instance = new Scheduler();
		}
		return *_instance;
	}

	bool register_with_rm();
	bool unregister_with_rm();

	bool RMisUP() {
		return _rm_is_up;
	}

	void RMisUP(bool rm_is_up) {
		_rm_is_up_lock.lock();
		_rm_is_up = rm_is_up;
		_rm_is_up_lock.unlock();
	}

	bool RMResetInProgress() {
		return _rm_reset_in_progress;
	}

	void RMResetInProgress(bool rm_reset_in_progress) {
		_rm_reset_in_progress_lock.lock();
		_rm_reset_in_progress = rm_reset_in_progress;
		_rm_reset_in_progress_lock.unlock();
	}

	void RMReset();
	void RMMarkDown();

	Machine* getMachine(char* machine_name);
	Job*     getJob(char* job_name);
	Step*    findStep(char* step_name);
	void     addStep(Step* step);

	void     deleteJob(char* job_name);
	void     deleteStep(char* step_name);

	EventListenerThread* eventListenerThread() {
		return _event_listener_thread;
	}

	void eventListenerThread(EventListenerThread* event_listener_thread);

	SchedulingThread* schedulingThread() {
		return _scheduling_thread;
	}

	void schedulingThread(SchedulingThread* scheduling_thread);

	static int   SCHEDULING_INTERVAL;
	static char* SCHEDULER_LOG;
	static int   DEBUG_LEVEL;

	Mutex lock;

	llr_resmgr_handle_t*     rm_handle;
	llr_event_registration_t reg_data;

	std::list<llr_event_t*>  event_queue;      // protected by event_queue_lock
	Mutex                    event_queue_lock; // protects event queue

	Mutex                    _rm_state_lock;   // guarantee that reset/mark_down are atomic

	Mutex                    steps_lock;
	Mutex                    jobs_lock;
	Mutex                    machines_lock;

	std::vector<Machine*>  machines;   // protected by lock
	std::vector<Job*>      jobs;       // protected by lock
	std::vector<Step*>     steps;      // protected by lock

private:

	Scheduler() : _rm_is_up(false), _rm_reset_in_progress(false), _scheduling_thread(NULL), _event_listener_thread(NULL) {}
	~Scheduler() {
		schedulingThread(NULL);
		eventListenerThread(NULL);
	}

	static Scheduler*     _instance;

	bool                  _rm_is_up;
	Mutex                 _rm_is_up_lock;

	bool                  _rm_reset_in_progress;
	Mutex                 _rm_reset_in_progress_lock;

	SchedulingThread*     _scheduling_thread;
	EventListenerThread*  _event_listener_thread;
};

#endif
