/* IBM_COPYRIGHT_BEGIN_TAG */
/*
 * Copyright IBM Corporation 2009. All rights reserved. The information contained
 * in these materials is provided for informational purposes only, and is provided AS IS
 * without warranty of any kind, express or implied. IBM shall not be responsible for any
 * damages arising out of the use of, or otherwise related to, these materials. Nothing
 * contained in these materials is intended to, nor shall have the effect of, creating any
 * warranties or representations from IBM or its suppliers or licensors, or altering the terms
 * and conditions of the applicable license agreement governing the use of IBM software.
 * References in these materials to IBM products, programs, or services do not imply that
 * they will be available in all countries in which IBM operates. Product release dates
 * and/or capabilities referenced in these materials may change at any time at IBM<92>s sole
 * discretion based on market opportunities or other factors, and are not intended to be a
 * commitment to future product or feature availability in any way. IBM, the IBM logo, AIX,
 * the AIX logo, and other IBM products and services are trademarks of the International
 * Business Machines Corporation, in the United States, other countries or both. Other
 * company, product, or service names may be trademarks or service marks of others.
 */
/* IBM_COPYRIGHT_END_TAG */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <errno.h>
#include <stdint.h> // for uint64_t
#include "llrapi.h"

main(int argc, char *argv[]) {

	llr_resmgr_handle_t *rm_handle;
	llr_element_t *err_object;
	llr_element_t *job = NULL;
	llr_job_info_t job_info;
	char *job_id;
	int rc;
	int step_count;
	char *path;
	int file_size;
	char *job_string;
	FILE *filep;

	if (argc != 2) {
		fprintf(stderr, "Usage: sample_submit <job command file>\n");
		exit(-1);
	}

	path = argv[1];

	filep = fopen(path, "r");
	if (filep == NULL) {
		fprintf(stderr, "Can not open file %s: %s\n",path, strerror(errno));
		exit(-1);
	}

	fseek (filep, 0, SEEK_END);
	file_size = ftell(filep);
	rewind (filep);

	job_string = (char*) malloc (file_size + 1);
	if (job_string == NULL) {
		fprintf(stderr, "Can not malloc %d bytes\n", file_size);
		fclose (filep);
		exit(-1);
	}

	rc = fread(job_string, 1, file_size, filep);
	if (rc != file_size) {
		fprintf(stderr, "Error read file %s: %s\n",path, strerror(errno));
		fclose (filep);
		exit(-1);
	}

	job_string[file_size] = '\0';
	fclose (filep);

	err_object = NULL;
	rc = llr_init_resmgr(LLR_API_VERSION, &rm_handle, &err_object);
	if (rc != LLR_API_OK) {
		fprintf(stderr, "Resource manager API can not be intialized.\n");
		if (err_object) {
			llr_error(&err_object, LLR_ERROR_PRINT_STDERR);
		}
		exit(-1);
	}

	job_info.format = LLR_LOADLEVELER_FORMAT;
	job_info.mode = LLR_BATCH_MODE;
	job_info.job_description = job_string;
	job_info.scheduler_id = "sample_scheduler";

	rc = llr_add_job(rm_handle, &job_info, &job, &err_object);
	if (rc != LLR_API_OK) {
		fprintf(stderr, "Job not submitted to the resource manager.\n");
		if (err_object) {
			llr_error(&err_object, LLR_ERROR_PRINT_STDERR);
		}
	} else {
		rc = llr_get_data(rm_handle, job, LLR_JobID, &job_id, &err_object);
		if (rc != LLR_API_OK) {
			if (err_object) {
				llr_error(&err_object, LLR_ERROR_PRINT_STDERR);
			}
		}

		if (rc == LLR_API_OK) {
			rc = llr_get_data(rm_handle, job, LLR_JobStepCount, &step_count, &err_object);
			if (rc != LLR_API_OK) {
				if (err_object) {
					llr_error(&err_object, LLR_ERROR_PRINT_STDERR);
				}
			}
		}

		fprintf(stdout, "Job %s with %d %s submitted:\n", job_id, step_count, step_count==1?"step":"steps");

		llr_data_list_t step_list;
		if (rc == LLR_API_OK) {
			rc = llr_get_data(rm_handle, job, LLR_JobGetStepList, &step_list, &err_object);
			if (rc != LLR_API_OK) {
				if (err_object) {
					llr_error(&err_object, LLR_ERROR_PRINT_STDERR);
				}
			}
		}

		void* step = NULL;
		if (rc == LLR_API_OK) {
			rc = llr_get_data(rm_handle, &step_list, LLR_JobGetFirstStep, &step, &err_object);
			if (rc != LLR_API_OK) {
				if (err_object) {
					llr_error(&err_object, LLR_ERROR_PRINT_STDERR);
				}
			}
		}

		while (step != NULL) {
			char* step_id = NULL;
			if (rc == LLR_API_OK) {
				char* step_id = NULL;
				rc = llr_get_data(rm_handle, step, LLR_StepID, &step_id, &err_object);
				if (rc != LLR_API_OK) {
					if (err_object) {
						llr_error(&err_object, LLR_ERROR_PRINT_STDERR);
					}
				} else {
					fprintf(stdout, "Job Step %s submitted.\n", step_id);
				}
			}

			if (step_id != NULL) free(step_id);

			if (rc == LLR_API_OK) {
				rc = llr_get_data(rm_handle, &step_list, LLR_JobGetNextStep, &step, &err_object);
				if (rc != LLR_API_OK) {
					if (err_object) {
						llr_error(&err_object, LLR_ERROR_PRINT_STDERR);
					}
				}
			}
		}

		llr_free_job(rm_handle, &job, &err_object);
	}

	llr_free_resmgr(&rm_handle, &err_object);
	free (job_string);
}

